( cd usr/lib ; rm -rf libgpg-error.so.0 )
( cd usr/lib ; ln -sf libgpg-error.so.0.4.0 libgpg-error.so.0 )
( cd usr/lib ; rm -rf libgpg-error.so )
( cd usr/lib ; ln -sf libgpg-error.so.0.4.0 libgpg-error.so )
